@@ -1,3 +1,6 @@
+extra.whitespace.pattern <- "^(\n|\r)+|(\t|\n|\r| )+$"
+
+
 .normalizeURL <- function (path)
 {
     # x <- "https://raw.githubusercontent.com////////////ArcadeAntics///testing/.././this.path/./main/tests/this.path_w_URLs.R"
@@ -5,20 +8,19 @@
     # source(x)
 
 
-    vapply(path.split(path), function(x) {
+    vapply(path.split.URL(path), function(x) {
         x <- x[x != "."]
         while (i <- match("..", x, nomatch = 0L)) {
             x <- if (i >= 3L)
                 x[-i + 0L:1L]
             else x[-i]
         }
-        x[-1L] <- gsub("^(\n|\r)+|(\t|\n|\r| )+$", "", x[-1L])
+        x[-1L] <- gsub(extra.whitespace.pattern, "", x[-1L])
         paste(x, collapse = "/")
     }, FUN.VALUE = "")
 }
 
 
-
 # normalizeURL <- function (path, mustWork = NA, against = NULL)
 # {
 #     if (is.character(against) || is.null(against)) {
